#include "DLLInterface.h"
#include "PackageBase.h"

#include <windows.h>

//////////////////////////////////////////////////////////////////////////
// DLL plumbing, in case we ever want to use it.
//////////////////////////////////////////////////////////////////////////

BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return TRUE;
}

//////////////////////////////////////////////////////////////////////////
// These routines are just C procedure interfaces to the identically
// named methods of your CUserDefComponentBase-derived class.
//////////////////////////////////////////////////////////////////////////

void RegisterCallbacks( CUserDefComponentBase* pObj, void* pSMOKObj, READMEMPORT_PROC pReadMemProc, WRITEMEMPORT_PROC pWriteMemProc )
{
	pObj->RegisterCallbacks( pSMOKObj, pReadMemProc, pWriteMemProc);
}

void GetPortDescription( CUserDefComponentBase* pObj, INPORTDESC** pInputPorts, OUTPORTDESC** pOutputPorts )
{
	pObj->GetPortDescriptionVectors( pInputPorts, pOutputPorts );
}

HALT_STATUS ComputeOutputs(CUserDefComponentBase* pObj, int* pInputArray, int* pOutputArray)
{
	return pObj->ComputeOutputs(pInputArray, pOutputArray);
}

HALT_STATUS LatchInputs(CUserDefComponentBase* pObj, int* pInputArray)
{
	return pObj->LatchInputs(pInputArray);
}

bool Reinitialize(CUserDefComponentBase* pObj)
{
	return pObj->Reinitialize();
}

void DeleteObj(CUserDefComponentBase* pObj)
{
	if (pObj) delete pObj;
}

//////////////////////////////////////////////////////////////////////////
// Basic package linkage routines.  GetPackageInformation() is called
// once, when the package is loaded.  GetDefinedComponentType() is called
// immediately thereafter, once for each component type.
//////////////////////////////////////////////////////////////////////////

bool GetDefinedComponentType( int typeIndex, const char** componentTypeName, 
							  CREATE_DEFINED_COMPONENT_PROC_TYPE* pCreateProc )
{
	*componentTypeName = componentTypes[typeIndex].componentName;
	*pCreateProc = componentTypes[typeIndex].pCreateObjectFunction;

	if ( !componentTypeName ) return false;
	return true;
}

bool GetPackageInformation( const char** pBaseVersion,  const char** pVersion,
							const char** pDate, const char** pAuthor, int* numTypes,
							GET_DEFINED_COMPONENT_TYPE_PROC* pGetTypeProc,
							REGISTERCALLBACKS_PROC* pRegisterCallbacksProc,
							GETPORTDESCRIPTION_PROC* pGetPortDescriptionProc,
							COMPUTEOUTPUT_PROC* pComputeOutputProc,
							LATCHINPUTS_PROC* pLatchInputsProc,
							REINITIALIZE_PROC* pReinitializeProc,
							DELETEOBJ_PROC* pDeleteProc)
{
	*pBaseVersion = PACKAGE_BASE_VERSION;	// version (major.minor) of this (base) code

	*pVersion = PACKAGE_VERSION;
	*pDate = PACKAGE_DATE;
	*pAuthor = PACKAGE_AUTHOR;

	for ( *numTypes=0; componentTypes[*numTypes].componentName; (*numTypes)++ );

	*pGetTypeProc = GetDefinedComponentType;
	*pRegisterCallbacksProc = RegisterCallbacks;
	*pGetPortDescriptionProc = GetPortDescription;
	*pComputeOutputProc = ComputeOutputs;
	*pLatchInputsProc = LatchInputs;
	*pReinitializeProc = Reinitialize;
	*pDeleteProc = DeleteObj;

	return true;
}

